@@ -1,9 +1,6 @@
 package IPC::Transit::Internal;
-{
-  $IPC::Transit::Internal::VERSION = '1.133280';
-}
-
-use strict;
+$IPC::Transit::Internal::VERSION = '1.161300';
+use strict;use warnings;
 use IPC::SysV;
 use IPC::Msg;
 use POSIX;
@@ -109,6 +106,12 @@ sub _gather_queue_info {
     }
 }
 
+sub _queue_exists {
+    my $qname = shift;
+    _mk_queue_dir();
+    return $config->{queues}->{$qname};
+}
+
 sub _get_queue_id {
     my %args = @_;
     _mk_queue_dir();
